from goonj.entities import AlertRuleConfig


def get_alert_rule_config(alert_rule_settings):
    alert_rule_config = {}
    if alert_rule_settings and len(alert_rule_settings) > 0:
        for alert_rule_setting in alert_rule_settings:
            frequency = None
            if 'frequency' in alert_rule_setting:
                frequency = alert_rule_setting['frequency']
            alert_rule_config[str(alert_rule_setting['error_code'])] = AlertRuleConfig(str(alert_rule_setting[
                'error_code']),
                alert_rule_setting[
                'threshold'],
                frequency)
    return alert_rule_config



import datetime

from goonj.conf.constants import Sev
from goonj.entities import CustomMessage, ErrorDetails
from goonj.rule_engine import RuleEngine


class SmartAlert(object):
    rule_engine = RuleEngine()

    def __init__(self, name, source, logger=None, alert_rule_config=None):
        """

        :param name: name of the source
        :param source:  value of the source
        :param logger: logger object used for general logging
        """
        self.source = source
        self.name = name
        self.logger = logger
        self.alert_rule_config = alert_rule_config
        self.error_details_map = {}

    def warn(self, message, sev=None, subject=None, error_id=None, error=None,
             tag_list=None, *args, **kwargs):
        self.log_and_alert_message('warn', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def info(self, message, sev=None, subject=None, error_id=None, error=None,
             tag_list=None, *args, **kwargs):
        self.log_and_alert_message('info', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def error(self, message, sev=None, subject=None, error_id=None, error=None,
              tag_list=None, *args, **kwargs):
        self.log_and_alert_message('error', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def debug(self, message, sev=None, subject=None, error_id=None, error=None,
              tag_list=None, *args, **kwargs):
        self.log_and_alert_message('debug', message, sev, subject, error_id, error,
                                   tag_list, *args, **kwargs)

    def exception(self, message, sev=None, subject=None, error_id=None, error=None,
                  tag_list=None, *args, **kwargs):
        self.log_and_alert_message('exception', message, sev, subject, error_id, error,

                                   tag_list, *args, **kwargs)

    def log_and_alert_message(self, function_name, message, sev=None, subject=None, error_id=None, error=None,
                              tag_list=None, *args, **kwargs):
        if self.logger is not None:

            if sev is None:
                sev_value = 'No Sev'
            else:
                sev_value = sev.value

            custom_message = CustomMessage(
                tag_list, sev_value, message, error, error_id, subject)
            getattr(self.logger, function_name)(
                custom_message, *args, **kwargs)

        self.__alert(sev, message, subject, error_id, error, tag_list)

    def __alert(self, sev, message, subject, error_id, error, tag_list):

        if not sev:
            return

        if not isinstance(sev, Sev):
            raise TypeError('Sev must be an instance of type Sev')

        if error_id in self.error_details_map:
            error_detail = self.error_details_map[error_id]
            error_detail.error_count = error_detail.error_count + 1
            error_detail.error_timestamp_list.append(datetime.datetime.now())

        else:
            error_timestamp_list = []
            self.error_details_map[error_id] = ErrorDetails(
                1, error_timestamp_list)

        if self.rule_engine and not self.rule_engine.is_alerting_required(error_id, self.error_details_map,
                                                                          self.alert_rule_config):
            return

        if self.source.default_channels:
            for channel in self.source.default_channels:
                channel.send(sev, message, subject, error_id, error, tag_list)

        try:
            if self.source.severity:
                severity = self.source.severity[sev.value]
                for channel in severity.channels:
                    channel.send(sev, message, subject,
                                 error_id, error, tag_list)
        except KeyError as e:
            pass
            # Can supresss this excpetion need to check behaviour
            # raise ChannelsNotDefinedForSev('Channels are not defined for {} '
            #                                'sev in config file'
            #                                ''.format(sev.value))



import datetime

from croniter import croniter


class RuleEngine(object):

    def is_time_valid(self, error_time, prev_alert_time):
        if error_time >= prev_alert_time:
            return True

    def filter_error_time_list_before_prev_alert_time(self, error_timestamp_list, prev_alert_time):
        error_timestamp_list[:] = [error_time for error_time in error_timestamp_list if self.is_time_valid(
            error_time, prev_alert_time)]

        return error_timestamp_list

    def get_prev_alert_time(self, frequency, base):
        iter = croniter(frequency, base)
        prev_alert_time = iter.get_prev(datetime.datetime)

        return prev_alert_time

    def is_alerting_required(self, error_code, error_detail_map, alert_rule_config):

        if alert_rule_config and error_code in alert_rule_config:

            frequency = alert_rule_config[error_code].frequency
            error_timestamp_list = error_detail_map[error_code].error_timestamp_list
            if frequency:
                base = datetime.datetime.now()
                prev_alert_time = self.get_prev_alert_time(frequency, base)

                self.filter_error_time_list_before_prev_alert_time(
                    error_timestamp_list, prev_alert_time)

            if error_timestamp_list.__len__() >= (alert_rule_config[error_code].threshold):
                error_detail_map[error_code].error_count = 0
                error_detail_map[error_code].error_timestamp_list = []
                return True
            else:
                return False

        return True
